var H5P = H5P || {};

H5P.KaTeX = (function ($) {

	var $ = H5P.jQuery;
	
	/**
	* Constructor function.
	*/
	function KaTeX(options, id) {
	};
	
	/**
	* Document ready function.
	*/
	$( document ).ready(function() {    
		// Define MutationObservers
		// This is the standard observer, if no other actions are required.
		var observer_standard = new MutationObserver(function(mutations) {
			console.log("Observer standard fired");
			mutations.forEach(function(mutation) {
				H5P.katex.renderMathInElement(mutation.target);
				console.log(mutation.target);
			});
		});
	
		// Check the Overlay Layer of Interactive Video for Changes
		// There is only one overlay Layer (no loop needed)
		var observer_overlay = new MutationObserver(function(mutations) {
			console.log("Observer Overlay fired");
			mutations.forEach(function(mutation) {
				H5P.katex.renderMathInElement(mutation.target);
				console.log(mutation.target);
			});
		
			// When some Question Types are shown there need to be additional Mutation Oberservers
			observeAnswers();
		});
			
		// Only render Math every other mutation, since the rendering is a mutation and only
		// the contents gets changed when answers change order (multiple choice question)
		var math_rendering = false;
		var observer_answers = new MutationObserver(function(mutations) {
			console.log("Observer answers fired");
			if (math_rendering == false) {
				mutations.forEach(function(mutation) {
					H5P.katex.renderMathInElement(mutation.target);
					console.log(mutation.target);
				});
				math_rendering = true;
			} else {
				math_rendering = false;
			}
		});

		// After editor change there are new Objects to observe
		var observer_editor = new MutationObserver(function(mutations) {
			console.log("Observer EditorChange fired");
			observeObjects();
			observeEditorObjects();
			var config = { attributes: false, childList: true, characterData: false };
			// h5peditor-interactions (child list changed on switch to step 2 of interactive video)
			// needed because the hook for subeditor changes is not present at interactive video editor load)
			$( ".h5peditor-interactions" ).each(function( index ) {
				observer_editor.observe($(this)[0], config);
				console.log("observing h5peditor-interactions");
				console.log($(this)[0]);
			});
		});
		
		var observer_editor_dragquestion = new MutationObserver(function(mutations) {
			console.log("Observer editor dragquestion fired");
			$( ".h5peditor-dragquestion" ).each(function( index ) {
				H5P.katex.renderMathInElement($(this)[0]);
			});
		});
		
		var observer_editor_questionset = new MutationObserver(function(mutations) {
			console.log("Observer editor questionset fired");
			var config = { attributes: true, childList: false, characterData: false };
			$( ".libwrap" ).each(function( index ) {
				observer_editor.observe($(this)[0], config);
				console.log("observing libwrap");
				console.log($(this)[0]);
			});
		});
		
		// Register the observers after initial loading of the page
		// It doesn't matter if it is actually the right H5P, because we are always checking
		// if the element exists at the moment.
		function observeAnswers() {
			var config = { attributes: true, childList: true, characterData: true };
			// Drag and Drop
			$( ".h5p-inner" ).each(function( index ) {
				observer_standard.observe($(this)[0], config);
				console.log("observing h5pinnner");
				console.log($(this)[0]);
			});
			// Multiple Choice
			$( ".h5p-alternative-inner" ).each(function( index ) {
				observer_answers.observe($(this)[0], config);
				console.log("observing h5p-alternative-inner");
				console.log($(this)[0]);
			});
			// Dialogcards
			var config = { attributes: false, childList: true, characterData: false };
			$( ".h5p-dialogcards-card-text-area" ).each(function( index ) {
				observer_answers.observe($(this)[0], config);
				console.log("observing h5p-dialogcards-card-text-area");
				console.log($(this)[0]);
			});
		}
		
		// Observe Objects that are only loaded in Editor Mode.
		function observeEditorObjects() {
			// Dragquestion
			var config = { attributes: false, childList: true, characterData: false };
			var config = { attributes: true, childList: false, characterData: false };
			$( ".h5peditor-dragquestion" ).each(function( index ) {
				observer_editor_dragquestion.observe($(this)[0], config);
				console.log("observing h5peditor-dragquestion");
				console.log($(this)[0]);
			});
			
			// Question set: 
			// h5p-vtab-forms childs for added questions
			// libwrap childs for changed (loaded) question type
			var config = { attributes: false, childList: true, characterData: false };
			if ($(".h5p-vtab-forms")[0]) {
				observer_editor_questionset.observe($(".h5p-vtab-forms")[0], config);
				console.log("observing h5p-vtab-forms");
				console.log($(this)[0]);
			}
			$( ".libwrap" ).each(function( index ) {
				observer_editor.observe($(this)[0], config);
				console.log("observing libwrap");
				console.log($(this)[0]);
			});
		}
	
		function observeObjects() {
			var config = { attributes: true, childList: true, characterData: false, subtree: false };
			// Overlay and Dialog in Interactive Video
			$( ".h5p-overlay" ).each(function( index ) {
				observer_overlay.observe($(this)[0], config);
				console.log("observing h5p-overlay");
				console.log($(this)[0]);
			});
			$( ".h5p-dialog-wrapper" ).each(function( index ) {
				observer_overlay.observe($(this)[0], config);
				console.log("observing h5p-dialog-wrapper");
				console.log($(this)[0]);
			});
			// QuestionSet
			$( ".question-container" ).each(function( index ) {
				observer_overlay.observe($(this)[0], config);
				console.log("observing question-container");
				console.log($(this)[0]);
			});
			// DragText
			var config = { attributes: false, childList: true, characterData: false, subtree: true };
			$( ".h5p-drag-draggables-container" ).each(function( index ) {
				observer_answers.observe($(this)[0], config);
				console.log("observing h5p-drag-draggables-container");
				console.log($(this)[0]);
			});
			$( ".h5p-drag-dropzone-container" ).each(function( index ) {
				observer_answers.observe($(this)[0], config);
				console.log("observing h5p-drag-dropzone-container");
				console.log($(this)[0]);
			});
			// Dialogcards
			var config = { attributes: false, childList: true, characterData: false };
			$( ".h5p-dialogcards-cardwrap-set" ).each(function( index ) {
				observer_overlay.observe($(this)[0], config);
				console.log("observing h5p-dialogcards-cardwrap-set");
				console.log($(this)[0]);
			});
			observeAnswers();
		}
		observeObjects();
	
		// Check if the Editor Changed, because the Elements only get generated then...
		var config = { attributes: true, childList: true, characterData: true };
		if ($(".h5peditor")[0]) {
			observer_editor.observe($(".h5peditor")[0], config);
			console.log("observing h5peditor");
			console.log($(this)[0]);
		}
		
		// Stuff we only need to do once, after load because other hooks handle the rest.
		$( ".h5p-content" ).each(function( index ) {
			H5P.katex.renderMathInElement($(".h5p-content")[0]);
		});
	});

  return KaTeX;
})(H5P.jQuery);

